/**
  *  \file gfx/blit.cc
  */

#include "gfx/blit.h"
#include "gfx/gfx-modetr.h"

template<typename Traits>
void
doBlitInternal(SDL_Surface* src, SDL_Surface* target, const GfxRect& src_clip,
               int x, int y, int x_dx, int x_dy, int y_dx, int y_dy)
{
    if (x_dy < 0 && y_dy > 0) {
        /* both axes point into different direction, as in:

                    . x_dx,x_dy
                   / \
                  /   \
                 /     \
            x,y o       \
                 \       o
                  \     /
                   \   /
                    \ /
                     ` y_dx,y_dy

           Do one sweep through the source pixmap along the X axis,
           and one along the Y axis. */
        for (int dy = 0; dy > y_dy; --dy) {
            int lx = x + dy*y_dx/y_dy;
            int ly = y + dy;
        }
        for (int dy = 1; dy < x_dy; ++dy) {
            
        }
    } else {
    }
}

/** Stretch/Rotate Blit.
    \param src        source surface
    \param target     target surface
    \param src_clip   area relative to src, the area to display
    \param x,y        position in target surface which will receive the first
                      pixel (i.e. src(src_clip.y,src_clip.x)
    \param x_dx,x_dy  X axis: from x,y to x+x_dx,x_dy
    \param y_dx,y_dy  Y axis: from x,y to x+y_dx,y_dy */
void
doBlit(SDL_Surface* src, SDL_Surface* target, const GfxRect& src_clip,
       int x, int y, int x_dx, int x_dy, int y_dx, int y_dy)
{
    if (src_clip.w == 0 || src_clip.h == 0)
        /* input is empty */
        return;
    if (x_dx == 0 && x_dy == 0)
        /* X axis has length zero */
        return;
    if (y_dx == 0 && y_dy == 0)
        /* Y axis has length zero */
        return;
    if (x_dx == 0 && y_dx == 0)
        /* X axis is parallel to Y axis, so height is zero */
        return;
    if (x_dy == 0 && y_dy == 0)
        /* X axis is parallel to Y axis, so width is zero */
        return;

    ASSERT(src->format->BitsPerPixel == target->format->BitsPerPixel);

    GFX_MODE_SWITCH(doBlitInternal, (src, target, src_clip, x, y, x_dx, x_dy, y_dx, y_dy));
}
