#include <iostream>
#include "game/cargo.h"
#include "cpluslib/format.h"
#include "util/console.h"

class TestObject : public GCargoContainer {
    GCargo cargo;
    int content[el_COUNT];
 public:
    TestObject();
    ~TestObject();

    int32  getAmount(CargoType type);
    void   changeAmount(CargoType type, int32 amount);
    void   reserveAmount(CargoType type, int32 amount);
    bool     canHave(CargoType cargo_id);
    int32    getMax(CargoType cargo_id);
    string_t getName() const;
    void dump(GCargoCostTransaction& act);
};

TestObject::TestObject()
    : cargo()
{
    cargo.incUseCount();
    for (int i = 0; i < el_COUNT; ++i)
        content[i] = 1000;
}

TestObject::~TestObject()
{
    cargo.decUseCount();
}

int32
TestObject::getAmount(CargoType type)
{
    return content[type] - cargo.getReserved(type);
}

void
TestObject::changeAmount(CargoType type, int32 amount)
{
    content[type] += amount;
    std::cout << "changed " << getCargoName(type)
              << " by " << amount << " to " << content[type] << "\n";
}

void
TestObject::reserveAmount(CargoType type, int32 amount)
{
    cargo.changeReservation(type, amount);
    std::cout << "changed reservation for " << getCargoName(type)
              << " by " << amount << " to " << cargo.getReserved(type) << "\n";
}

bool
TestObject::canHave(CargoType cargo_id)
{
    return true;
}

int32
TestObject::getMax(CargoType cargo_id)
{
    return 1000000;
}

string_t
TestObject::getName() const
{
    return string_t("Test Object");
}

void
TestObject::dump(GCargoCostTransaction& act)
{
    std::cout << "Type                  Have   Resvd    Cost   Remain Miss\n"
              << "--------------------  -----  -----    -----  -----  -----\n";
    for (int i = 0; i < el_Torps; ++i) {
        GCargoType t = GCargoType(i);
        if (t != el_Tritanium && t != el_Molybdenum && t != el_Duranium
            && t != el_Supplies && t != el_Money)
            continue;
        std::cout << (format("%-16s (%d): %5d  %5d    %5d  %5d  %5d\n")
                      << getCargoName(t)
                      << i
                      << content[t]
                      << cargo.getReserved(t)
                      << act.getCost(t)
                      << act.getRemaining(t)
                      << act.getMissing(t));
    }
}

class TestTA : public GCargoCostTransaction {
 public:
    TestTA(TestObject& con);
};

TestTA::TestTA(TestObject& con)
    : GCargoCostTransaction(con)
{
    std::cout << "(q)uit (c)ommit (d)isplay (a)ddcost (s)ubcost\n";
    char c;
    int n, x;
    while (cin.get(c)) {
        switch (c) {
         case 'q':
            cancel();
            con.cancel();
            return;
         case 'c':
            std::cout << "commit...\n";
            commit();
            con.commit();
            return;
         case 'd':
            con.dump(*this);
            if (isValid())
                std::cout << "-- VALID --\n";
            else
                std::cout << "-- INVALID --\n";
            break;
         case 'a':
            cin >> n >> x;
            changeCost(GCargoType(n), x);
            break;
         case 's':
            cin >> n >> x;
            changeCost(GCargoType(n), -x);
            break;
        }
    }
    cancel();
    con.cancel();    
}

int main()
{
    enableStdioConsole();
    TestObject t;
    TestTA ta(t);
}
